﻿using MyTrelloWeb.App.AppServices.Common;
using MyTrelloWeb.App.Dto;
using MyTrelloWeb.App.Entity;
using MyTrelloWeb.App.EntityFramework;
using System;
using System.Linq;

namespace MyTrelloWeb.App.AppServices
{
    public interface IKanbanService
    {
        // kanban level

        ListSummaryOutput[] KanbanSummary(); //TODO: KanbanId

        // list level

        ListSummaryOutput ListSummary(QueryByIdInput input);

        KanbanEntryCreateOutput ListCreate(CreateWithNameInput input); //TODO: Kanban ID

        void ListRename(ContentInput input);

        void ListDelete(DeleteByIdInput input);

        void ListMove(ListMoveInput input);

        // card level

        CardDetailOutput CardDetail(QueryByIdInput input);

        KanbanEntryCreateOutput CardCreate(CreateWithNameAndParentIdInput input);

        void CardRename(ContentInput input);

        void CardEditSummary(ContentInput input);

        void CardMove(MoveInput input);

        void CardDelete(DeleteByIdInput input);

        // manifest level

        KanbanEntryCreateOutput ManifestCreate(CreateWithNameAndParentIdInput input);

        void ManifestMove(MoveInput input);

        void ManifestRename(ContentInput input);

        void ManifestDelete(DeleteByIdInput input);

        // item level

        KanbanEntryCreateOutput ItemCreate(CreateWithNameAndParentIdInput input);

        void ItemMove(MoveInput input);

        void ItemRename(ContentInput input);

        void ItemDelete(DeleteByIdInput input);

        void ItemCheck(CheckInput input);
    }

    internal class KanbanService : AppServiceBase<KanbanEntity>, IKanbanService
    {
        ListSummaryOutput[] IKanbanService.KanbanSummary()
        {
            KanbanEntity[] all;
            using (var repo = GetRepository())
            {
                all = repo.Query().ToArray();
            }

            var lists = all.Where(p => KanbanEntryType.List == p.Type);

            return lists.Select(l =>
            {
                var cards = all.Where(c => KanbanEntryType.Card == c.Type && c.ListId == l.Id).ToArray();

                return new ListSummaryOutput
                {
                    Id = l.Id,
                    Sort = l.Sort,
                    Name = l.Name,
                    Cards = cards.Select(c =>
                    {
                        var checks = all.Where(i => i.Type == KanbanEntryType.Item && i.CardId == c.Id).ToArray();
                        return new CardSummaryOutput
                        {
                            Id = c.Id,
                            Sort = c.Sort,
                            Name = c.Name,
                            Summary = c.Summary,
                            Items = checks.Length,
                            ItemsChecked = checks.Count(p => p.Checked)
                        };
                    }).ToArray(),
                };
            }).ToArray();
        }

        ListSummaryOutput IKanbanService.ListSummary(QueryByIdInput input)
        {
            KanbanEntity[] all;
            using (var repo = GetRepository())
            {
                all = repo.Query(p =>
                    p.ListId == input.Id
                ).ToArray();
            }

            var l = all.First(p => p.Id == input.Id);

            var cards = all.Where(c => KanbanEntryType.Card == c.Type && c.ListId == l.Id).ToArray();

            return new ListSummaryOutput
            {
                Id = l.Id,
                Sort = l.Sort,
                Name = l.Name,
                Cards = cards.Select(c =>
                {
                    var checks = all.Where(i => i.Type == KanbanEntryType.Item && i.CardId == c.Id).ToArray();
                    return new CardSummaryOutput
                    {
                        Id = c.Id,
                        Sort = c.Sort,
                        Name = c.Name,
                        Summary = c.Summary,
                        Items = checks.Length,
                        ItemsChecked = checks.Count(p => p.Checked)
                    };
                }).ToArray(),
            };
        }

        KanbanEntryCreateOutput IKanbanService.ListCreate(CreateWithNameInput input)
        {
            var ne = new KanbanEntity
            {
                //TODO: Kanban ID
                Type = KanbanEntryType.List,
                Name = input.Name,
            };
            ne.ListId = ne.Id;

            using (var repo = GetRepository())
            {
                var existedCount = repo.Query(p => p.Type == KanbanEntryType.List).Count();
                ne.Sort = existedCount + 1;
                repo.Add(ne);
                repo.SaveChanges();
            }

            return new KanbanEntryCreateOutput
            {
                Id = ne.Id,
                Sort = ne.Sort
            };
        }

        void IKanbanService.ListRename(ContentInput input)
        {
            using (var repo = GetRepository())
            {
                var ce = repo
                    .Query(p => p.Id == input.Id && p.Type == KanbanEntryType.List)
                    .FirstOrThrowNotFound("列表");

                ce.Name = input.Content;
                repo.SaveChanges();
            }
        }

        void IKanbanService.ListDelete(DeleteByIdInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
            using (var repo = GetRepository())
            {
                var ce = repo
                    .Query(p => p.Id == input.Id && p.Type == KanbanEntryType.List)
                    .FirstOrThrowNotFound("列表");

                repo.Delete(ce);
                repo.SaveChanges();
            }
        }

        void IKanbanService.ListMove(ListMoveInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
        }

        CardDetailOutput IKanbanService.CardDetail(QueryByIdInput input)
        {
            throw new NotImplementedException();
        }

        KanbanEntryCreateOutput IKanbanService.CardCreate(CreateWithNameAndParentIdInput input)
        {
            KanbanEntity ne;

            using (var repo = GetRepository())
            {
                var olist = repo
                    .Query(p => p.Type == KanbanEntryType.List && p.Id == input.ParentId)
                    .FirstOrThrowNotFound("列表");

                ne = new KanbanEntity
                {
                    //TODO:  KanbanId
                    Type = KanbanEntryType.Card,
                    ListId = input.ParentId,
                    Name = input.Name,
                };
                ne.CardId = ne.Id;

                var existedCount = repo.Query(p => p.Type == KanbanEntryType.Card && p.ListId == input.ParentId).Count();
                ne.Sort = existedCount + 1;
                repo.Add(ne);
                repo.SaveChanges();
            }

            return new KanbanEntryCreateOutput
            {
                Id = ne.Id,
                Sort = ne.Sort
            };
        }

        void IKanbanService.CardRename(ContentInput input)
        {
            using (var repo = GetRepository())
            {
                var ce = repo
                    .Query(p => p.Id == input.Id && p.Type == KanbanEntryType.Card)
                    .FirstOrThrowNotFound("卡片");

                ce.Name = input.Content;
                repo.SaveChanges();
            }
        }

        void IKanbanService.CardEditSummary(ContentInput input)
        {
            using (var repo = GetRepository())
            {
                var ce = repo
                    .Query(p => p.Id == input.Id && p.Type == KanbanEntryType.Card)
                    .FirstOrThrowNotFound("卡片");

                ce.Summary = input.Content;
                repo.SaveChanges();
            }
        }

        void IKanbanService.CardMove(MoveInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
        }

        void IKanbanService.CardDelete(DeleteByIdInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
        }

        KanbanEntryCreateOutput IKanbanService.ManifestCreate(CreateWithNameAndParentIdInput input)
        {
            throw new NotImplementedException();
        }

        void IKanbanService.ManifestMove(MoveInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
        }

        void IKanbanService.ManifestRename(ContentInput input)
        {
            throw new NotImplementedException();
        }

        void IKanbanService.ManifestDelete(DeleteByIdInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
        }

        KanbanEntryCreateOutput IKanbanService.ItemCreate(CreateWithNameAndParentIdInput input)
        {
            throw new NotImplementedException();
        }

        void IKanbanService.ItemMove(MoveInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
        }

        void IKanbanService.ItemRename(ContentInput input)
        {
            throw new NotImplementedException();
        }

        void IKanbanService.ItemDelete(DeleteByIdInput input)
        {
            //TODO: Re Sort
            throw new NotImplementedException();
        }

        void IKanbanService.ItemCheck(CheckInput input)
        {
            throw new NotImplementedException();
        }
    }
}
