﻿using FluentMigrator;
using MyTrelloWeb.VCommon.Logging;
using MyTrelloWeb.VCommon.VEntity;

namespace MyTrelloWeb.App.DbMigration
{
    [Migration(Version)]
    public class InitDatabase : Migration
    {
        private const long Version = 20190413235130;

        public override void Up()
        {
            Logger.Info($"Exec Migration UP {Version}_{GetType().Name}");

            Create.Table("Kanban")
                .WithColumn("Id").AsGuid().PrimaryKey()
                .WithColumn(nameof(ISoftDelete.IsAbolish)).AsBoolean().Indexed()
                .WithColumn("Type").AsByte().Indexed()
                .WithColumn("ListId").AsGuid().Indexed()
                .WithColumn("CardId").AsGuid().Indexed()
                .WithColumn("ManifestId").AsGuid().Indexed()
                .WithColumn("Sort").AsInt32()
                .WithColumn("Title").AsString(128)
                .WithColumn("Summary").AsString(4096).Nullable()
                .WithColumn("Checked").AsBoolean()
                .WithColumn(nameof(IHaveCreationTime.CreationTime)).AsDateTime()
                .WithColumn(nameof(IHaveLastModificationTime.LastModificationTime)).AsDateTime().Nullable()
                ;
        }

        public override void Down()
        {
            Logger.Info($"Exec Migration DOWN {Version}_{GetType().Name}");

            Delete.Table("Kanban");
        }
    }
}
