﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Reflection;
using MyTrelloWeb.VCommon.DataAnnotations.Auditing;
using MyTrelloWeb.VCommon.Dto;
using MyTrelloWeb.VCommon.Ioc;
using MyTrelloWeb.VCommon.Logging;
using MyTrelloWeb.VCommon.Reflection;

namespace MyTrelloWeb.VCommon.VApplication.Auditing
{
    public abstract class ApplicationServiceAuditInterceptorBase : InterfaceInterceptorBase, ITransientIocClass
    {
        //field

        //TODO: 需要授权再打开
        //private readonly IVSession _session;

        private Guid? _beforeTenantId;
        private Guid? _beforeUserId;

        //TODO: 需要授权再打开
        //protected ApplicationServiceAuditInterceptorBase(IVSession session) => _session = session;
        protected ApplicationServiceAuditInterceptorBase()
        {
        }

        //built-in PreProcess Intercepter

        protected static void InputValidationIntercept(IReadOnlyDictionary<string, object> paramDic)
        {
            var vPass = true;
            var vResult = new List<ValidationResult>();
            foreach (var kvArg in paramDic.Where(p => null != p.Value))
            {
                var objToValidation = kvArg.Value;
                var vctx = new ValidationContext(objToValidation);
                //validation by system
                vPass &= Validator.TryValidateObject(objToValidation, vctx, vResult,true);
                //validation by custom implement if has
                if (objToValidation is ICustomValidation cv) vPass &= cv.CustomValidation(vResult);
            }

            //collect results and throw
            if (false == vPass)
            {
                throw new VApplicationModelValidationException("模型校验失败", vResult);
            }
        }

        protected void AuthPermissionIntercept(DateTime time, Guid? tenantId, Guid? userId, Type svcClass, MethodBase svcMethod, IReadOnlyDictionary<string, object> paramDic)
        {
            //TODO: 需要授权再打开
            //var svcAuthInClass = svcClass.GetCustomAttributeIncludeInterface<VServiceAuthorizeAttribute>();
            //if (0 != svcAuthInClass.Length) _session.DemandAuth();

            //var svcAuthInMethod = svcMethod.GetCustomAttribute<VServiceAuthorizeAttribute>(true);
            //if (null != svcAuthInMethod) _session.DemandAuth();

            //if (false == IocManager.Instance.IsRegistered<IPermissionManager>()) return;

            //if (0 == svcAuthInClass.Length && null == svcAuthInMethod) return;

            //if (0 == svcAuthInClass.Sum(p => p.AnyPermissionsRequired.Length)
            //    && 0 == (svcAuthInMethod?.AnyPermissionsRequired.Length ?? 0)) return;

            //var permMgr = IocManager.Instance.Resolve<IPermissionManager>();

            //if ((0 == svcAuthInClass.Length || permMgr.CheckPermission(_session.GetTenantId(), _session.GetUserId(), svcAuthInClass.SelectMany(p => p.AnyPermissionsRequired).ToArray())) &&
            //    (null == svcAuthInMethod || permMgr.CheckPermission(_session.GetTenantId(), _session.GetUserId(), svcAuthInMethod.AnyPermissionsRequired))) return;

            //var vrwFriendlyException = new VApplicationAuthException("拒绝访问:您没有权限进行此操作", AuthReason.AccessDenied);

            //PreWriteAuditLog(time, -1, tenantId, userId, svcClass, svcMethod, paramDic, null, vrwFriendlyException);

            //throw vrwFriendlyException;
        }

        protected void AuthOrgIntercept(DateTime time, Guid? tenantId, Guid? userId, Type svcClass, MethodBase svcMethod, IReadOnlyDictionary<string, object> paramDic)
        {
            //TODO: 需要授权再打开
            //var mustHaveOrgs = paramDic.Values.OfType<IMustHaveOrg>().ToArray();
            //if (mustHaveOrgs.Length == 0) return;

            //if (false == IocManager.Instance.IsRegistered<IOrgManager>()) return;

            //var orgMgr = IocManager.Instance.Resolve<IOrgManager>();
            //if (false == orgMgr.Enabled) return;

            //var uvOrgs = orgMgr.GetUserVisionOrgs(_session.GetTenantId(), _session.GetUserId()).ToArray();
            //if (mustHaveOrgs.All(argument => argument.OrgId.In(uvOrgs))) return;

            //var vrwFriendlyException = new VApplicationAuthException("拒绝访问:您没有权限操作此对象", AuthReason.AccessDenied);

            //PreWriteAuditLog(time, -1, tenantId, userId, svcClass, svcMethod, paramDic, null, vrwFriendlyException);

            //throw vrwFriendlyException;
        }

        //built-in PostProcess LogAction

        protected void LogServiceInvoke(DateTime time, long msUsed, Guid? tenantIdBefore, Guid? userIdBefore, Guid? tenantIdAfter, Guid? userIdAfter, Type svcClass, MethodBase svcMethod, IReadOnlyDictionary<string, object> paramDic, object returnValue, Exception exception)
        {
            object ex = exception;

            var tenantIdToLog = tenantIdAfter ?? tenantIdBefore;
            var userIdToLog = userIdAfter ?? userIdBefore;

            //如果非登录或退出时(即有一边为null)用户标识变化,那就见鬼了,要记下来
            if (null != tenantIdBefore && null != tenantIdAfter && tenantIdBefore != tenantIdAfter
                || null != userIdBefore && null != userIdAfter && userIdBefore != userIdAfter)
            {
                Logger.Warn("Unexcepted user id change", new
                {
                    tenantIdBefore,
                    tenantIdAfter,
                    userIdBefore,
                    userIdAfter
                });
            }

            PreWriteAuditLog(time, msUsed, tenantIdToLog, userIdToLog, svcClass, svcMethod, paramDic, returnValue, ex);
        }

        //ovr able

        protected override void BeforeInvoke(DateTime time, Type svcClass, MethodBase svcMethod, IReadOnlyDictionary<string, object> paramDic)
        {
            //TODO: 需要授权再打开
            _beforeTenantId = Guid.Empty;// _session.TenantId;
            _beforeUserId = Guid.Empty;// _session.UserId;

            BeforeInvokeService(time, _beforeTenantId, _beforeUserId, svcClass, svcMethod, paramDic);
        }

        protected override void AfterInvoke(DateTime time, long msUsed, Type svcClass, MethodBase svcMethod, IReadOnlyDictionary<string, object> paramDic, object returnValue, Exception exception)
        {
            AfterInvokeService(time, msUsed, _beforeTenantId, _beforeUserId
                //TODO: 需要授权再打开
                , Guid.Empty // _session.TenantId
                , Guid.Empty// _session.UserId
                , svcClass, svcMethod, paramDic, returnValue, exception);
        }

        public virtual void BeforeInvokeService(DateTime time, Guid? tenantId, Guid? userId, Type svcClass, MethodBase svcMethod, IReadOnlyDictionary<string, object> paramDic)
        {
            InputValidationIntercept(paramDic);
            AuthPermissionIntercept(time, tenantId, userId, svcClass, svcMethod, paramDic);
            AuthOrgIntercept(time, tenantId, userId, svcClass, svcMethod, paramDic);
        }

        public virtual void AfterInvokeService(DateTime time, long msUsed, Guid? tenantIdBefore, Guid? userIdBefore, Guid? tenantIdAfter, Guid? userIdAfter, Type svcClass, MethodBase svcMethod, IReadOnlyDictionary<string, object> paramDic, object returnValue, Exception exception)
        {
            LogServiceInvoke(time, msUsed, tenantIdBefore, userIdBefore, tenantIdAfter, userIdAfter, svcClass, svcMethod, paramDic, returnValue, exception);
        }

        protected virtual void PreWriteAuditLog(DateTime time, long msUsed, Guid? tenantId, Guid? userId, Type serviceClass, MethodBase serviceMethod, IReadOnlyDictionary<string, object> input, object output, object exception)
        {
            if (serviceClass.IsDefinedIncludeInterface(typeof(DisableServiceAuditingLogAttribute)) || serviceMethod.IsDefined(typeof(DisableServiceAuditingLogAttribute))) return;

            WriteAuditLog(time, msUsed, tenantId, userId, serviceClass.FullName, serviceMethod.Name, input, output, exception);
        }

        //abstract

        protected abstract void WriteAuditLog(DateTime time, long msUsed, Guid? tenantId, Guid? userId, string serviceClassName, string serviceMethodName, IReadOnlyDictionary<string, object> input, object output, object exception);
    }
}
