﻿using System;
using System.Data.Entity;
using System.Linq;
using EntityFramework.DynamicFilters;
using MyTrelloWeb.VCommon.VEntity;

namespace MyTrelloWeb.VCommon.VEntityFramework
{
    public abstract class VEfDbContextBase : DbContext
    {
        protected VEfDbContextBase(string nameOrConnectionString) : base(nameOrConnectionString)
        {
        }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            base.OnModelCreating(modelBuilder);

            modelBuilder.Filter(VEfDataFilters.SoftDelete, (ISoftDelete d) => d.IsAbolish, false);
            modelBuilder.Filter(VEfDataFilters.Passive, (IPassave d) => d.IsEnable, true);
        }

        public override int SaveChanges()
        {
            foreach (var track in ChangeTracker.Entries<IHaveCreationTime>().Where(p => p.State == EntityState.Added)) track.Entity.CreationTime = DateTime.Now;
            foreach (var track in ChangeTracker.Entries<IHaveLastModificationTime>().Where(p => p.State == EntityState.Modified)) track.Entity.LastModificationTime = DateTime.Now;

            foreach (var track in ChangeTracker.Entries<ISoftDelete>().Where(p => p.State == EntityState.Deleted))
            {
                track.State = EntityState.Modified;
                track.Entity.IsAbolish = true;

                if (track.Entity is IHaveLastModificationTime lmd) lmd.LastModificationTime = DateTime.Now;
            }

            return base.SaveChanges();
        }

        public virtual void EnableFilter(string filterName) => DynamicFilterExtensions.EnableFilter(this, filterName);

        public void DisableFilter(string filterName) => DynamicFilterExtensions.DisableFilter(this, filterName);

        public void EnableAllFilters() => DynamicFilterExtensions.EnableAllFilters(this);

        public void DisableAllFilters() => DynamicFilterExtensions.DisableAllFilters(this);

        public bool IsFilterEnabled(string filterName) => DynamicFilterExtensions.IsFilterEnabled(this, filterName);
    }
}
