﻿using System;
using System.Data.Entity;
using System.Linq;
using System.Linq.Expressions;
using MyTrelloWeb.VCommon.VAutoMapper;
using MyTrelloWeb.VCommon.VEntity;

namespace MyTrelloWeb.VCommon.VEntityFramework
{
    public abstract class VEfRepositoryBase<TDbContext, TEntity> : IDisposable where TEntity : VEntityBase where TDbContext : VEfDbContextBase
    {
        private TDbContext _dbContext;

        public TDbContext DbContext
        {
            get => _dbContext ?? (_dbContext = ResolveDbContextInstance());
            set => _dbContext = value;
        }

        public abstract VEfRepositoryBase<TDbContext, TForkEntity> ForkRepository<TForkEntity>() where TForkEntity : VEntityBase;

        protected abstract TDbContext ResolveDbContextInstance();

        private DbSet<TEntity> DbSet => DbContext.Set<TEntity>();

        public void DisableFilter(string filterName) => DbContext.DisableFilter(filterName);

        public IDisposable DisableFilterHolding(string filterName) => new FilterDisableState<TDbContext>(DbContext, filterName);

        public IQueryable<TEntity> QueryNoTracking(Expression<Func<TEntity, bool>> queryExpression = null) => queryExpression != null
            ? DbSet.AsNoTracking().Where(queryExpression)
            : DbSet.AsNoTracking();

        public TProjection[] QueryProjection<TProjection>(Expression<Func<TEntity, bool>> queryExpression = null) => queryExpression != null
            ? DbSet.Where(queryExpression).ProjectionToArray<TEntity, TProjection>()
            : DbSet.ProjectionToArray<TEntity, TProjection>();

        public TProjection QueryProjection<TProjection>(Guid id) where TProjection : class => DbSet.Where(p => p.Id == id).ProjectionToArray<TEntity, TProjection>().FirstOrDefault();

        public IQueryable<TEntity> Query(Expression<Func<TEntity, bool>> queryExpression = null) => null == queryExpression ? DbSet : DbSet.Where(queryExpression);

        public bool CheckExist(Expression<Func<TEntity, bool>> expression = null) => null == expression ? DbSet.Any() : DbSet.Any(expression);

        public TEntity GetEntntyOrDefault(Guid id) => DbContext.Set<TEntity>().GetOrDefault(id);

        public TEntity GetEntntyOrDefault(Expression<Func<TEntity, bool>> expression = null) => null == expression ? DbContext.Set<TEntity>().FirstOrDefault() : DbContext.Set<TEntity>().GetOrDefault(expression);

        public TEntity Add(TEntity entnty) => DbSet.Add(entnty);

        public void Delete(TEntity entnty) => DbSet.Remove(entnty);

        public int SaveChanges() => DbContext.SaveChanges();

        public void Dispose() => DbContext?.Dispose();
    }
}