﻿using System;
using System.IO;
using System.Reflection;
using System.Text;
using MyTrelloWeb.VCommon.Json;
using MyTrelloWeb.VCommon.Logging;
using MyTrelloWeb.VCommon.VOpenApi.Json;
using Newtonsoft.Json;

namespace MyTrelloWeb.VCommon.VOpenApi
{
    public class ApiBind
    {
        private static readonly JsonReturnValueSerializer ReturnValueSerializer = new JsonReturnValueSerializer();

        public string RawRoute { get; }

        private readonly Func<object> _resolveFunc;
        public Type ApiResourceType { get; }
        public MethodInfo ApiMethod { get; }
        public Type InputParamType { get; }

        public string ResourceName { get; }
        public string Name { get; }

        public ApiBind(MethodInfo apiMethod, string resourceName, string name, Func<object> resolveFunc, string rawRoute)
        {
            ApiMethod = apiMethod;
            ResourceName = resourceName;

            Name = name;
            ApiResourceType = apiMethod.DeclaringType;

            var parameters = ApiMethod.GetParameters();
            if (1 == parameters.Length)
            {
                InputParamType = parameters[0].ParameterType;
            }

            _resolveFunc = resolveFunc;
            RawRoute = rawRoute;
        }

        public void Invoke(Stream inputStream, Stream outputStream)
        {
            object inputParam = null;
            if (null != InputParamType)
            {
                //TODO: PACT? Scheme check?

                var inputReader = new StreamReader(inputStream, Encoding.UTF8);
                var input = inputReader.ReadToEnd();

                try
                {
                    inputParam = VJsonSerializer.Deserialize(input, InputParamType);
                }
                catch (JsonReaderException exception)
                {
                    throw new VApiArgumentException("参数错误:" + exception.Message);
                }
                catch (JsonSerializationException exception)
                {
                    throw new VApiArgumentException($"参数错误:{exception.Message}{exception.InnerException?.Message}");
                }
                catch (Exception exception)
                {
                    Logger.Error("ApiBind: Error on parsing input", new { inputJson = input, exception });
                    throw new VApiArgumentException("参数错误");
                }

                if (null == inputParam) throw new VApiArgumentException("缺少参数");
            }

            var inst = _resolveFunc();

            //TODO: 使用更高性能的动态调用方式
            var returnValue = ApiResourceType.InvokeMember(
                ApiMethod.Name, BindingFlags.InvokeMethod, null, inst,
                inputParam == null ? null : new[] { inputParam }
            );

            var output = ReturnValueSerializer.SerializeObject(returnValue);
            var bufOutput = Encoding.UTF8.GetBytes(output);
            outputStream.Write(bufOutput, 0, bufOutput.Length);
        }
    }
}
