﻿using System;
using System.IO;
using System.Web;
using MyTrelloWeb.VCommon.Json;
using MyTrelloWeb.VCommon.Logging;

namespace MyTrelloWeb.VCommon.VSwaggerUI
{
    public abstract class SwaggerUiModuleBase : IHttpModule
    {
        private readonly ISwaggerUiResourceProvider _resourceProvider;

        protected virtual bool Enable => true;

        private readonly string _basePath;

        protected SwaggerUiModuleBase(ISwaggerUiResourceProvider resourceProvider, string basePath = "/swagger")
        {
            _resourceProvider = resourceProvider;
            _basePath = basePath;
        }

        public void Init(HttpApplication context)
        {
            if (Enable) context.BeginRequest += Context_BeginRequest;
        }

        public void Dispose()
        {
            if (false == Enable) return;
        }

        private void Context_BeginRequest(object sender, EventArgs e)
        {
            try
            {
                var requestPath = HttpContext.Current.Request.Path.ToLower();
                if (!requestPath.StartsWith(_basePath)) return;

                var path = requestPath.Substring(_basePath.Length);

                if (path == "/")
                {
                    var api = HttpUtility.UrlDecode(HttpContext.Current.Request.QueryString.ToString());
                    var html = _resourceProvider.GetIndex(api);

                    //var html = System.Text.Encoding.UTF8.GetString(resourceProvider.GetBinResource("index.html"));
                    //var requestUrl = HttpContext.Current.Request.Url;

                    //html = html.Replace("$(HOST)", requestUrl.GetLeftPart(UriPartial.Scheme | UriPartial.Authority));
                    //html = html.Replace("$(API_PATH_WITHOUT_ROOT)", api);

                    HttpContext.Current.Response.Write(html);
                }
                else
                {
                    var fileName = Path.GetFileName(path);

                    var bin = _resourceProvider.GetBinResource(fileName);
                    if (null == bin)
                    {
                        HttpContext.Current.Response.StatusCode = 404;
                    }
                    else
                    {
                        if (Path.GetExtension(fileName).ToLower() == ".css")
                            HttpContext.Current.Response.ContentType = "text/css";
                        HttpContext.Current.Response.BinaryWrite(bin);
                    }
                }
            }
            catch (Exception exception)
            {
                Logger.Error("SwaggerUiModuleBase: exception during handle request", exception);
                if (HttpContext.Current.IsDebuggingEnabled)
                {
                    HttpContext.Current.Response.ContentType = "application/json";
                    HttpContext.Current.Response.Write(VJsonSerializer.Serialize(exception));
                }
                else
                {
                    HttpContext.Current.Response.ContentType = "text/plain";
                    HttpContext.Current.Response.Write("Server internal error");
                }
            }
            HttpContext.Current.Response.Flush();
            HttpContext.Current.Response.SuppressContent = true;
            HttpContext.Current.ApplicationInstance.CompleteRequest();
        }
    }
}